/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.tools.common.views;


import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.drawable.Drawable;
import android.text.TextUtils;
import android.util.AttributeSet;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.LinearLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.appcompat.widget.AppCompatTextView;
import androidx.appcompat.widget.PopupMenu;
import androidx.constraintlayout.widget.ConstraintLayout;

import com.compdfkit.tools.R;
import com.compdfkit.tools.common.utils.viewutils.CViewUtils;

import java.util.ArrayList;
import java.util.List;


/**
 * pdf ui tool bar <br/>
 * ︳------------------------------------------------︳<br/>
 * ︳　　　　　　　　title　　　　　icon1  icon2 icon3   ︳<br/>
 * ︳------------------------------------------------︳<br/>
 * icon1: searchIcon <br/>
 * icon2: boTaIcon <br/>
 * icon3: moreIcon <br/>
 * <p/>
 * use samples:<br/>
 * com.compdfkit.tools.common.utils.view.CPDFToolBar <br/>
 * android:id="@+id/pdf_tool_bar" <br/>
 * android:layout_width="match_parent" <br/>
 * android:layout_height="?android:attr/actionBarSize" <br/>
 * app:tools_toolbar_title="@string/viewer_toolbar_title" <br/>
 * app:tools_toolbar_bota_icon="@drawable/xxx" <br/>
 * app:tools_toolbar_more_icon="@drawable/xxx" <br/>
 * app:tools_toolbar_search_icon="@drawable/xxx"/> <br/>
 * <p/>
 * custom attrs: <br/>
 * app:tools_toolbar_title="@string/xxx" <br/>
 * app:tools_toolbar_search_icon="@drawable/xxx" <br/>
 * app:tools_toolbar_bota_icon="@drawable/xxx" <br/>
 * app:tools_toolbar_more_icon="@drawable/xxx" <br/>
 * <p/>
 * btn click listener
 * searchBtnClick:
 *
 * @see CPDFToolBar#setSearchBtnClickListener(OnClickListener)
 * <p>
 * botaBtnClick:
 * @see CPDFToolBar#setBoTaBtnClickListener(OnClickListener)
 * <p>
 * moreBtnClick:
 * @see CPDFToolBar#setMoreBtnClickListener(OnClickListener)
 */
public class CPDFToolBar extends FrameLayout implements PopupMenu.OnMenuItemClickListener {



    enum PreviewMode {
        PDFView,
        Annotation,
        Form,
        Edit,
        PageEdit
    }

    private AppCompatTextView tvToolBarTitle;

    private AppCompatImageView ivToolBarSearchBtn;

    private AppCompatImageView ivToolBarBoTaBtn;

    private AppCompatImageView ivToolBarMoreBtn;

    private AppCompatImageView ivToolBarThumbnail;

    private ConstraintLayout clMainToolbar;

    private AppCompatImageView ivTitleArrow;

    private LinearLayout llTitle;

    private PopupMenu popupMenu;

    private List<PreviewMode> previewModes = new ArrayList<>();
    private int samplePDFViewMenuId = View.generateViewId();
    private int annotViewMenuId = View.generateViewId();
    private int editMenuId = View.generateViewId();
    private int pageEditMenuId = View.generateViewId();
    private int formMenuId = View.generateViewId();

    private PreviewMode currentPreviewMode = PreviewMode.PDFView;

    private OnPreviewModeChangeListener changeListener;

    public CPDFToolBar(@NonNull Context context) {
        this(context, null);
    }

    public CPDFToolBar(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CPDFToolBar(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initToolBar(context, attrs);
    }

    private void initToolBar(Context context, @Nullable AttributeSet attrs) {
        LayoutInflater.from(getContext()).inflate(R.layout.tools_cpdf_tool_bar, this);
        clMainToolbar = findViewById(R.id.cl_pdf_root_tool_bar);
        tvToolBarTitle = findViewById(R.id.tv_tool_bar_title);
        ivToolBarMoreBtn = findViewById(R.id.iv_tool_bar_more);
        ivToolBarSearchBtn = findViewById(R.id.iv_tool_bar_search);
        ivToolBarBoTaBtn = findViewById(R.id.iv_tool_bar_bota);
        ivToolBarThumbnail = findViewById(R.id.iv_tool_bar_thumbnail);
        llTitle = findViewById(R.id.ll_title);
        ivTitleArrow = findViewById(R.id.iv_down_arrow);
        popupMenu = new PopupMenu(getContext(), llTitle, Gravity.CENTER);
        modifyPopupMenuStatus();
        llTitle.setOnClickListener(v -> {
            PopupMenu popupMenu = new PopupMenu(getContext(), llTitle, Gravity.CENTER);

        });
        initAttributes(context, attrs);
    }

    private void initAttributes(Context context, @Nullable AttributeSet attrs) {
        TypedArray typedArray = context.obtainStyledAttributes(attrs, R.styleable.CPDFToolBar);
        if (typedArray != null) {
            String toolBarTitle = typedArray.getString(R.styleable.CPDFToolBar_tools_toolbar_title);
            typedArray.getResourceId(R.styleable.CPDFToolBar_tools_toolbar_search_icon, -1);
            if (!TextUtils.isEmpty(toolBarTitle)) {
                tvToolBarTitle.setText(toolBarTitle);
            }
            Drawable searchIconDrawable = CViewUtils.loadDrawableFromAttributes(getContext(), typedArray, R.styleable.CPDFToolBar_tools_toolbar_search_icon, R.drawable.tools_ic_search);
            if (searchIconDrawable != null) {
                ivToolBarSearchBtn.setImageDrawable(searchIconDrawable);
            }
            Drawable boTaIconDrawable = CViewUtils.loadDrawableFromAttributes(getContext(), typedArray, R.styleable.CPDFToolBar_tools_toolbar_bota_icon, R.drawable.tools_ic_outline);
            if (boTaIconDrawable != null) {
                ivToolBarBoTaBtn.setImageDrawable(boTaIconDrawable);
            }
            Drawable moreIconDrawable = CViewUtils.loadDrawableFromAttributes(getContext(), typedArray, R.styleable.CPDFToolBar_tools_toolbar_more_icon, R.drawable.tools_ic_more);
            if (moreIconDrawable != null) {
                ivToolBarMoreBtn.setImageDrawable(moreIconDrawable);
            }
            CViewUtils.applyViewBackground(this);
            typedArray.recycle();
        }
    }

    private void modifyPopupMenuStatus(){
        ivTitleArrow.setVisibility(previewModes.size() <=0 ? GONE : VISIBLE);
        popupMenu.getMenu().clear();
        for (int i = 0; i < previewModes.size(); i++) {
            PreviewMode type = previewModes.get(i);
            int titleResId;
            int menuId;
            switch(type){
                case Edit:
                    titleResId = R.string.tools_menu_edit;
                    menuId = editMenuId;
                    break;
                case Form:
                    titleResId = R.string.tools_menu_form;
                    menuId = formMenuId;
                    break;
                case PageEdit:
                    titleResId = R.string.tools_menu_page_edit;
                    menuId = pageEditMenuId;
                    break;
                case Annotation:
                    titleResId = R.string.tools_menu_annotation;
                    menuId = annotViewMenuId;
                    break;
                default:
                    titleResId = R.string.tools_menu_pdfview;
                    menuId = samplePDFViewMenuId;
                    break;
            }
            popupMenu.getMenu().add(Menu.NONE, menuId, i, getContext().getString(titleResId));
        }
    }


    @Override
    public boolean onMenuItemClick(MenuItem item) {
        if (item.getItemId() == samplePDFViewMenuId){

        } else if (item.getItemId() == annotViewMenuId) {

        } else if (item.getItemId() == editMenuId) {

        } else if (item.getItemId() == pageEditMenuId) {

        } else if (item.getItemId() == formMenuId){

        }
        return false;
    }

    private void changeMode(PreviewMode mode){
        currentPreviewMode = mode;
        if (changeListener != null) {
            changeListener.change(currentPreviewMode);
        }
    }

    public void setSearchBtnClickListener(View.OnClickListener clickListener) {
        ivToolBarSearchBtn.setOnClickListener(clickListener);
    }

    public void setBoTaBtnClickListener(View.OnClickListener clickListener) {
        ivToolBarBoTaBtn.setOnClickListener(clickListener);
    }

    public void setMoreBtnClickListener(View.OnClickListener clickListener) {
        ivToolBarMoreBtn.setOnClickListener(clickListener);
    }

    public void setThumbnailBtnClickListener(View.OnClickListener clickListener){
        ivToolBarThumbnail.setOnClickListener(clickListener);
    }

    public void setPreviewModeChangeListener(OnPreviewModeChangeListener changeListener) {
        this.changeListener = changeListener;
    }

    public void addMode(PreviewMode previewMode){
        previewModes.add(previewMode);
        modifyPopupMenuStatus();
    }

    public interface OnPreviewModeChangeListener{
        void change(PreviewMode mode);
    }


}
